#include "string_view2.hxx"
#include <algorithm>
#include <cstring>

string_view::string_view(const char* begin, const char* end)
    : begin_(begin), end_(end)
{
    if (begin > end)
        throw std::invalid_argument("string_view: begin > end");
}

string_view::string_view(const char* s, size_t size)
    : string_view(s, s + size)
{ }

string_view::string_view(Owned_string const& s)
    : string_view(s.begin(), s.end())
{ }

string_view::string_view(const char* s)
    : string_view(s, std::strlen(s))
{ }

size_t string_view::size() const
{
    return end_ - begin_;
}

const char* string_view::begin() const
{
    return begin_;
}

const char* string_view::end() const
{
    return end_;
}

bool operator==(string_view sv1, string_view sv2)
{
    return sv1.size() == sv2.size() &&
        std::equal(sv1.begin(), sv1.end(), sv2.begin());
}

bool operator!=(string_view sv1, string_view sv2)
{
    return !(sv1 == sv2);
}

std::ostream& operator<<(std::ostream& os, string_view sv)
{
    return os.write(sv.begin(), sv.size());
}

